<?php

namespace Drupal\custom_book_block;

use Drupal\book\BookManager;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\Core\Language\LanguageInterface;
use Drupal\node\NodeInterface;

// Override BookManager service.

class expandBookManager extends BookManager {

  /**
   * {@inheritdoc}
   */
  public function bookTreeAllData($bid, $link = NULL, $max_depth = NULL, $always_expand = 0) {
    $tree = &drupal_static(__METHOD__, []);
    $language_interface = \Drupal::languageManager()->getCurrentLanguage();

    // Use $nid as a flag for whether the data being loaded is for the whole
    // tree.
    $nid = isset($link['nid']) ? $link['nid'] : 0;
    // Generate a cache ID (cid) specific for this $bid, $link, $language, and
    // depth.
    $cid = 'book-links:' . $bid . ':all:' . $nid . ':' . $language_interface->getId() . ':' . (int) $max_depth;

    if (!isset($tree[$cid])) {
      // If the tree data was not in the static cache, build $tree_parameters.
      $tree_parameters = [
        'min_depth' => 1,
        'max_depth' => $max_depth,
      ];
      if ($nid) {
        $active_trail = $this->getActiveTrailIds($bid, $link);
        // Setting the 'expanded' value to $active_trail would be same as core.
        if ($always_expand) {
          $tree_parameters['expanded'] = [];
        }
        else {
          $tree_parameters['expanded'] = $active_trail;
        }
        $tree_parameters['active_trail'] = $active_trail;
        $tree_parameters['active_trail'][] = $nid;
      }

      // Build the tree using the parameters; the resulting tree will be cached.
      $tree[$cid] = $this->bookTreeBuild($bid, $tree_parameters);
    }

    return $tree[$cid];
  }

  /**
   * Builds the #items property for a book tree's renderable array.
   *
   * Helper function for ::bookTreeOutput().
   *
   * @param array $tree
   *   A data structure representing the tree.
   *
   * @return array
   *   The value to use for the #items property of a renderable menu.
   */
  protected function buildItems(array $tree) {
    $items = [];
    $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $node = \Drupal::routeMatch()->getParameter('node');

    foreach ($tree as $data) {
      $element = [];
      $inner_node = \Drupal::entityTypeManager()->getStorage('node')->load($data['link']['nid']);

      // Generally we only deal with visible links, but just in case.
      if (!$data['link']['access']) {
        continue;
      }
      // Set a class for the <li> tag. Since $data['below'] may contain local
      // tasks, only set 'expanded' to true if the link also has children within
      // the current book.
      $element['is_expanded'] = FALSE;
      $element['is_collapsed'] = FALSE;
      if ($data['link']['has_children'] && $data['below']) {
        $element['is_expanded'] = TRUE;
      }
      elseif ($data['link']['has_children']) {
        $element['is_collapsed'] = TRUE;
      }

      // Set a helper variable to indicate if the link is in the active trail.
      $element['in_active_trail'] = FALSE;
      if ($data['link']['in_active_trail']) {
        $element['in_active_trail'] = TRUE;
      }

      // Allow book-specific theme overrides.
      $element['attributes'] = new Attribute();
      $element['title'] = $data['link']['title'];

      // If the node is not translated then make the path <front>.
      if ($inner_node instanceof NodeInterface) {
        if (!$inner_node->hasTranslation($langcode)) {
          $element['url'] =  Url::fromRoute('<front>');
        }
        else {
          $element['url'] = Url::fromUri('entity:node/' . $data['link']['nid'], [
            'language' => $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT),
          ]);
        }
      }

      $element['in_active_trail'] = FALSE;
      if ($node instanceof NodeInterface) {
        if ($data['link']['nid'] == $node->id()) {
          $element['in_active_trail'] = TRUE;
        }
      }

      $element['localized_options'] = !empty($data['link']['localized_options']) ? $data['link']['localized_options'] : [];
      $element['localized_options']['set_active_class'] = TRUE;
      $element['below'] = $data['below'] ? $this->buildItems($data['below']) : [];
      $element['original_link'] = $data['link'];
      // Index using the link's unique nid.
      $items[$data['link']['nid']] = $element;
    }

    return $items;
  }

}
